/*	$Id: derivnum.c,v 3.3 1998/06/01 16:10:08 kas Exp $	*/

/* The numerically computed gradient (using finite differences) */

/* This file was originally written by Petr Mejzlk */

/*
 *  This file is a part of Opal. See the Opal www page at
 *  http://www.fi.muni.cz/~kas/opal for details.
 *
 *  Copyright (C) 1995-1998  Jan "Yenya" Kasprzak <kas@fi.muni.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "proto.h"
#include "ff.h"
#include "pval.h"

#define max(A, B) ((A) > (B) ? (A) : (B))
#define DET(a11,a12,a21,a22) ((a11)*(a22)-(a21)*(a12))

static void solve3lin(double a[3][3], double b[], double x[])
/* solve a regular well-conditioned system of three linear equations ax=b */
{
	double det;
	int i,j;

	det = 0.0;
	for(i=0;i<3;i++) det += a[i][0] * DET(a[(i+1)%3][1], a[(i+1)%3][2],
					      a[(i+2)%3][1], a[(i+2)%3][2]);
	for(j=0;j<3;j++) {
		x[j] = 0.0;
		for(i=0;i<3;i++)
		  x[j] += b[i] * DET(a[(i+1)%3][(j+1)%3], a[(i+1)%3][(j+2)%3],
				     a[(i+2)%3][(j+1)%3], a[(i+2)%3][(j+2)%3]);
		x[j] /= det;
	}
}

void compute_grad_num(pvp)
potential_value *pvp;
{
	int i, dim;
        double a[3][3], b[3], x[3], value;
        int natom;
        double nx, ny, nz;
	static double delta=0.05;
	dim = 3*pvp->ffp->natoms;
	b[0] = compute_potential(pvp);
	for(i=0;i<dim;i++) {
		a[0][0] = pvp->x[i]*pvp->x[i]; a[0][1] = pvp->x[i];
		a[0][2] = a[1][2] = a[2][2] = 1.0;
		natom = i % pvp->ffp->natoms; 
		/* natom = number of the atom that moves */
		pvp->x[i] -= delta;
		nx = pvp->x[natom]; ny =  pvp->y[natom]; nz = pvp->z[natom];
		value = recompute_potential(pvp, natom, nx, ny, nz);
		b[1] = value; a[1][0] = pvp->x[i]*pvp->x[i]; a[1][1] = pvp->x[i];
		pvp->x[i] += 2.0*delta;
		nx = pvp->x[natom]; ny =  pvp->y[natom]; nz = pvp->z[natom];
		value = recompute_potential(pvp, natom, nx, ny, nz);
		b[2] = value; a[2][0] = pvp->x[i]*pvp->x[i]; a[2][1] = pvp->x[i];
		pvp->x[i] -= delta;
		solve3lin(a,b,x);

		pvp->dx[i] = 2*x[0]*pvp->x[i]+x[1];
	}
}
