/* $Id: dihprm.c,v 3.2 1998/06/01 16:10:08 kas Exp $ */

/* Program prints out the parameters of the dihedral angles in the .top file */

/*
 *  This file is a part of Opal. See the Opal www page at
 *  http://www.fi.muni.cz/~kas/opal for details.
 *
 *  Copyright (C) 1995-1998  Jan "Yenya" Kasprzak <kas@fi.muni.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <string.h>
#include "prm.h"

extern struct parm *readparm __P((char *filename));

#define atomname(p,x)	(p->AtomNames+4*p->x)

int find_bond(p, a1, a2)
struct parm *p;
int a1, a2;
{
	int i;
	int h=1;
#ifdef DEBUG
	printf("Looking for %d-%d bond ...\n", a1,a2);
#endif
	for (i=0; i<p->Nbonh; i++)
		if ((p->BondHAt1[i] == a1
			&& p->BondHAt2[i] == a2)
			|| (p->BondHAt2[i] == a1
			&& p->BondHAt1[i] == a2))
			goto found;
	h=0;
	for (i=0; i<p->Nbona; i++)
		if ((p->BondAt1[i] == a1
			&& p->BondAt2[i] == a2)
			|| (p->BondAt2[i] == a1
			&& p->BondAt1[i] == a2))
			goto found;
	printf("FATAL: Bond parameters for %d-%d not found!\n", a1, a2);
found:	
	return h ? p->BondHNum[i] : p->BondNum[i];
}

int find_angle(p, a1, a2, a3)
struct parm *p;
int a1, a2, a3;
{
	int i;
	int h=1;
	for (i=0; i<p->Ntheth; i++)
		if ((p->AngleHAt2[i] == a2) &&
			((p->AngleHAt1[i] == a1
			&& p->AngleHAt3[i] == a3)
			|| (p->AngleHAt3[i] == a1
			&& p->AngleHAt1[i] == a3)))
			goto found;
	h=0;
	for (i=0; i<p->Ntheta; i++)
		if ((p->AngleAt2[i] == a2) &&
			((p->AngleAt1[i] == a1
			&& p->AngleAt3[i] == a3)
			|| (p->AngleAt3[i] == a1
			&& p->AngleAt1[i] == a3)))
			goto found;
	printf("FATAL: Angle parameters for %d-%d-%d not found!\n", a1, a2, a3);
found:
	return h ? p->AngleHNum[i] : p->AngleNum[i];
}

static void usage(str)
char *str;
{
	fprintf(stderr, "Usage: %s foobar.top\n", str);
	exit(1);
}

int main(argc, argv)
int argc;
char **argv;
{
	struct parm *prm;
	int i;

	if (argc != 2)
		usage(*argv);
		
	prm = readparm(argv[1]);

	for (i=0; i<prm->Nphia; i++)
	{
		printf("%2d: %.3s-%.3s-%.3s-%.3s (%2d-%2d-%2d-%2d) type=%2d, Pk=%2.2f, "
			"Pn=%2.2f, Phase=%2.2f%s%s\n",
			i,
			atomname(prm, DihAt1[i]), atomname(prm, DihAt2[i]),
			atomname(prm, DihAt3[i]), atomname(prm, DihAt4[i]),
			prm->DihAt1[i]+1, prm->DihAt2[i]+1,
			prm->DihAt3[i]+1, prm->DihAt4[i]+1,
			prm->DihNum[i], 
			prm->Pk[prm->DihNum[i]],
			prm->Pn[prm->DihNum[i]],
			prm->Phase[prm->DihNum[i]],
			prm->DihImp[i] ? ", improper" : "",
			prm->DihEign[i] ? ", end_ign" : ""
			);
		if (prm->DihImp[i])
			printf ("\tReqs: length(1-3)=%2.2f, length(2-3)=%2.2f,"
				" length(3-4)=%2.2f\n",
				prm->Req[find_bond(prm, prm->DihAt1[i],
				prm->DihAt3[i])],
				prm->Req[find_bond(prm, prm->DihAt2[i],
				prm->DihAt3[i])],
				prm->Req[find_bond(prm, prm->DihAt3[i],
				prm->DihAt4[i])]);
		else
			printf ("\tReqs: length(1-2)=%2.2f, length(2-3)=%2.2f,"
				" length(3-4)=%2.2f\n",
				prm->Req[find_bond(prm, prm->DihAt1[i],
				prm->DihAt2[i])],
				prm->Req[find_bond(prm, prm->DihAt2[i],
				prm->DihAt3[i])],
				prm->Req[find_bond(prm, prm->DihAt3[i],
				prm->DihAt4[i])]);
		if (prm->DihImp[i])
			printf("\tTeqs: Angle(1-3-2)=%2.2f, Angle(2-3-4)=%2.2f\n",
				prm->Teq[find_angle(prm, prm->DihAt1[i],
				prm->DihAt3[i],
				prm->DihAt2[i])],
				prm->Teq[find_angle(prm, prm->DihAt2[i],
				prm->DihAt3[i],
				prm->DihAt4[i])]);
		else
			printf("\tTeqs: Angle(1-2-3)=%2.2f, Angle(2-3-4)=%2.2f\n",
				prm->Teq[find_angle(prm, prm->DihAt1[i],
				prm->DihAt2[i],
				prm->DihAt3[i])],
				prm->Teq[find_angle(prm, prm->DihAt2[i],
				prm->DihAt3[i],
				prm->DihAt4[i])]);
	}
	for (i=0; i<prm->Nphih; i++)
	{
		printf("%2d: %.3s-%.3s-%.3s-%.3s (%2d-%2d-%2d-%2d) type=%2d, Pk=%2.2f, "
			"Pn=%2.2f, Phase=%2.2f%s%s\n",
			i,
			atomname(prm, DihHAt1[i]), atomname(prm, DihHAt2[i]),
			atomname(prm, DihHAt3[i]), atomname(prm, DihHAt4[i]),
			prm->DihHAt1[i]+1, prm->DihHAt2[i]+1,
			prm->DihHAt3[i]+1, prm->DihHAt4[i]+1,
			prm->DihHNum[i], 
			prm->Pk[prm->DihHNum[i]],
			prm->Pn[prm->DihHNum[i]],
			prm->Phase[prm->DihHNum[i]],
			prm->DihHImp[i] ? ", improper" : "",
			prm->DihHEign[i] ? ", end_ign" : ""
			);
		if (prm->DihHImp[i])
			printf ("\tReqs: Length(1-3)=%2.2f, Length(2-3)=%2.2f,"
				" Length(3-4)=%2.2f\n",
				prm->Req[find_bond(prm, prm->DihHAt1[i],
				prm->DihHAt3[i])],
				prm->Req[find_bond(prm, prm->DihHAt2[i],
				prm->DihHAt3[i])],
				prm->Req[find_bond(prm, prm->DihHAt3[i],
				prm->DihHAt4[i])]);
		else
			printf ("\tReqs: Length(1-2)=%2.2f, Length(2-3)=%2.2f,"
				" Length(3-4)=%2.2f\n",
				prm->Req[find_bond(prm, prm->DihHAt1[i],
				prm->DihHAt2[i])],
				prm->Req[find_bond(prm, prm->DihHAt2[i],
				prm->DihHAt3[i])],
				prm->Req[find_bond(prm, prm->DihHAt3[i],
				prm->DihHAt4[i])]);
		if (prm->DihHImp[i])
			printf("\tTeqs: Angle(1-3-2)=%2.2f, Angle(2-3-4)=%2.2f\n",
				prm->Teq[find_angle(prm, prm->DihHAt1[i],
				prm->DihHAt3[i],
				prm->DihHAt2[i])],
				prm->Teq[find_angle(prm, prm->DihHAt2[i],
				prm->DihHAt3[i],
				prm->DihHAt4[i])]);
		else
			printf("\tTeqs: Angle(1-2-3)=%2.2f, Angle(2-3-4)=%2.2f\n",
				prm->Teq[find_angle(prm, prm->DihHAt1[i],
				prm->DihHAt2[i],
				prm->DihHAt3[i])],
				prm->Teq[find_angle(prm, prm->DihHAt2[i],
				prm->DihHAt3[i],
				prm->DihHAt4[i])]);
	}
	return 0;
}

