/* $Id: prm2ff.c,v 3.7 1998/06/01 16:10:15 kas Exp $ */

/* Converts the struct parm to the struct force_field. */

/*
 *  This file is a part of Opal. See the Opal www page at
 *  http://www.fi.muni.cz/~kas/opal for details.
 *
 *  Copyright (C) 1995-1998  Jan "Yenya" Kasprzak <kas@fi.muni.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdlib.h>
#include <string.h>

#include "ff.h"
#include "prm.h"

force_field *prm2ff(fnm)
char *fnm;
{
	force_field *ffp;
	struct parm *prm;
	int i, j;

	if (!(prm = readparm(fnm)))
		return NULL;
	
	ffp = malloc(sizeof(force_field));
	prm->ititl[80] = '\0';	/* Just in case ... */
	ffp->title = strdup(prm->ititl);

	ffp->natoms = prm->Natom;
	ffp->nresidues = prm->Nres;
	ffp->nbonds = prm->Nbonh + prm->Nbona;
	ffp->nangles = prm->Ntheth + prm->Ntheta;
	ffp->ndihs = prm->Nphih + prm->Nphia;

	ffp->natypes = prm->Ntypes;
	ffp->nhydronb = prm->Nphb;
	ffp->nbondtypes = prm->Numbnd;
	ffp->nangletypes = prm->Numang;
	ffp->ndihtypes = prm->Nptra;

	malloc_ff(ffp);

	for(i=0; i<ffp->natoms; i++)
	{
		strncpy(ffp->anames[i], prm->AtomNames+4*i, 4);
		ffp->anames[i][ATOM_NAME_LENGTH-1] = '\0';
		strncpy(ffp->atypenames[i], prm->AtomSym+4*i, 4);
		ffp->atypenames[i][ATOM_NAME_LENGTH-1] = '\0';

		ffp->atypes[i] = prm->Iac[i];
		ffp->charge[i] = prm->Charges[i];
/*		printf("DEBUG: atype[%d] = %d, name %s, type %s, charge %3.3f\n",
			i, ffp->atypes[i],
			ffp->anames[i], ffp->atypenames[i],
			ffp->charge[i]);
*/
	}

	for(i=0; i<ffp->nresidues; i++)
		ffp->residue[i] = prm->Ipres[i+1]-1;
#ifdef DEBUG_RESIDUE
	for (i=0; i<ffp->nresidues; i++)
		printf("residuum %3d ending at %3d\n", i, ffp->residue[i]);
#endif

	for (i=0; i<ffp->natypes*(ffp->natypes+1)/2; i++)
	{
		ffp->r12[i] = prm->Cn1[i];
		ffp->r6[i] = prm->Cn2[i];
	}

	for (i=0; i<ffp->nhydronb; i++)
	{
/*		printf("ffp->nhydronb=%d\n", ffp->nhydronb); */
		ffp->h12[i] = prm->HB12[i];
		ffp->h10[i] = prm->HB6[i];
	}

	for (i=0; i<ffp->natypes*ffp->natypes; i++)
	{
		ffp->nbnum[i] = prm->Cno[i];
/*		printf("DEBUG: nbnum[%d] = %d\n", i, ffp->nbnum[i]); */
	}

	for (i=0; i<ffp->nbondtypes; i++)
	{
		ffp->rk[i] = prm->Rk[i];
		ffp->req[i] = prm->Req[i];
	}

	for (i=0; i<ffp->nangletypes; i++)
	{
		ffp->tk[i] = prm->Tk[i];
		ffp->teq[i] = prm->Teq[i];
	}

	for (i=0; i<ffp->ndihtypes; i++)
	{
		ffp->pk[i] = prm->Pk[i];
		ffp->pn[i] = (int)prm->Pn[i];
		ffp->phase[i] = prm->Phase[i] == 0.0 ? 1 : -1;
	}
	
	for (i=j=0; j<prm->Nbonh; i++,j++)
	{
		ffp->bondat1[i] = prm->BondHAt1[j];
		ffp->bondat2[i] = prm->BondHAt2[j];
		ffp->bondnum[i] = prm->BondHNum[j];
	}

	for (j=0; j<prm->Nbona; i++,j++)
	{
		ffp->bondat1[i] = prm->BondAt1[j];
		ffp->bondat2[i] = prm->BondAt2[j];
		ffp->bondnum[i] = prm->BondNum[j];
	}

	for (i=j=0; j<prm->Ntheth; i++,j++)
	{
		ffp->angleat1[i] = prm->AngleHAt1[j];
		ffp->angleat2[i] = prm->AngleHAt2[j];
		ffp->angleat3[i] = prm->AngleHAt3[j];
		ffp->anglenum[i] = prm->AngleHNum[j];
	}

	for (j=0; j<prm->Ntheta; i++,j++)
	{
		ffp->angleat1[i] = prm->AngleAt1[j];
		ffp->angleat2[i] = prm->AngleAt2[j];
		ffp->angleat3[i] = prm->AngleAt3[j];
		ffp->anglenum[i] = prm->AngleNum[j];
	}

	for (i=j=0; j<prm->Nphih; i++,j++)
	{
		ffp->dihat1[i] = prm->DihHAt1[j];
		ffp->dihat2[i] = prm->DihHAt2[j];
		ffp->dihat3[i] = prm->DihHAt3[j];
		ffp->dihat4[i] = prm->DihHAt4[j];
		ffp->dihnum[i] = prm->DihHNum[j];
		ffp->improper[i] = prm->DihHImp[j];
		ffp->ignored[i] = prm->DihHEign[j] || ffp->improper[i];
	}
	for (j=0; j<prm->Nphia; i++,j++)
	{
		ffp->dihat1[i] = prm->DihAt1[j];
		ffp->dihat2[i] = prm->DihAt2[j];
		ffp->dihat3[i] = prm->DihAt3[j];
		ffp->dihat4[i] = prm->DihAt4[j];
		ffp->dihnum[i] = prm->DihNum[j];
		ffp->improper[i] = prm->DihImp[j];
		ffp->ignored[i] = prm->DihEign[j] || ffp->improper[i];
	}

	freeparm(prm);
	return ffp;
}

